﻿namespace QuickTemplate
open System
open System.ComponentModel.Composition
open System.Windows
open MoNo
open MoNo.Geometries
open MoNo.Studio
open MoNo.OpenGL


type [<Export (typeof<Studio.IMenuFactory>)>] private OpenGLMenuFactory() =
  inherit Studio.MenuFactory<QuickAppContext>()

  override __.CreateMenuItems con =
    let items =
      [
        // OpenGL を P/Invoke で直接呼び出して直線を描画
        MenuItem.New ("OpenGL Direct Call",
          cont {
            // MoNo.Graphics.IScene のインスタンスを生成
            let scene =
              { new Graphics.IScene with
                  member __.Draw sc =
                    // MoNo.OpenGL.GL クラスを利用すると P/Invoke により直接 OpenGL の API を呼び出すことができます。
                    GL.glDisable( GL.GL_LIGHTING )  // ライティングを無効化
                    GL.glColor3f( 1.f, 1.f, 0.f )   // 色を設定
                    GL.glBegin( GL.GL_LINES ) 
                    GL.glVertex3f( 0.f, 0.f, 0.f )
                    GL.glVertex3f( 1.f, 0.f, 0.f )
                    GL.glEnd() }
            con.Document.Val.Entries.Add (Wpf.Entry scene) // シーンオブジェクトを登録
          } |> Wpf.Cont.toWpfCommand con None)

        // MGL を利用して直線を描画
        MenuItem.New ("By MoNo.OpenGL.MGL",
          cont {
            // MoNo.Graphics.IScene のインスタンスを生成
            let scene =
              { new Graphics.IScene with
                  member __.Draw sc =
                    MGL.LightingEnabled <- false
                    MGL.Color <- System.Drawing.Color.Pink.ToColor4f()
                    use gl = MGL.Begin( GLPrimType.Lines )
                    gl.Vertex( Point3d.Zero ) 
                    gl.Vertex( Point3d( 1.0, 1.0, 1.0 ) )  }
            con.Document.Val.Entries.Add (Wpf.Entry scene) // シーンオブジェクトを登録
          } |> Wpf.Cont.toWpfCommand con None)

        // ISceneContext を利用して直線を描画
        MenuItem.New ("By MoNo.Graphics.ISceneContext",
          cont {
            // MoNo.Graphics.IScene インターフェイスのインスタンスを生成します
            let scene =
              { new Graphics.IScene with
                  member __.Draw sc =
                    use scope = sc.Push() // scope を利用すると Lighting 等の状態変更が using スコープ内に限定されます
                    scope.Lighting <- false
                    scope.Color <- System.Drawing.Color.Plum
                    sc.DrawLines (fun gl -> gl.Vertices( Point3d.Zero, Point3d( 1.0, 2.0, 3.0 ) ) ) }
            con.Document.Val.Entries.Add (Wpf.Entry scene) // シーンオブジェクトを登録
          } |> Wpf.Cont.toWpfCommand con None)

        // 頂点配列による折線描画（MGL）
        MenuItem.New ("VertexArray (MoNo.OpenGL.MGL)",
          cont {
            // 頂点配列を作成
            let array =
              [|
                Point3d( 0.0, 0.0, 0.0 )
                Point3d( 0.2, 0.0, 0.0 )
                Point3d( 0.2, 0.2, 0.0 )
                Point3d( 0.0, 0.2, 0.0 )
                Point3d( 0.0, 0.4, 0.0 )
                Point3d( 0.2, 0.4, 0.0 )
                Point3d( 0.2, 0.6, 0.0 )
                Point3d( 0.0, 0.6, 0.0 )
              |]
            // MoNo.Graphics.IScene のインスタンス
            let scene =
              { new Graphics.IScene with
                  member __.Draw sc =
                    use scope = sc.Push()
                    scope.Lighting <- false
                    scope.Color <- System.Drawing.Color.Salmon
                    MGL.DrawArrays (GLPrimType.LineStrip, array) }
            // シーンオブジェクトを登録
            con.Document.Val.Entries.Add (Wpf.Entry scene)
          } |> Wpf.Cont.toWpfCommand con None)

        // 頂点配列による折線描画（GraphicsUT）
        MenuItem.New ("VertexArray (MoNo.GraphicsUT)",
          cont {
            // 頂点配列を作成
            let array =
              GraphicsUT.CreateScene (GLPrimType.LineStrip,
                [|
                  Point3d( 0.0, 0.0, 0.0 )
                  Point3d( 0.2, 0.0, 0.0 )
                  Point3d( 0.2, 0.2, 0.0 )
                  Point3d( 0.0, 0.2, 0.0 )
                  Point3d( 0.0, 0.4, 0.0 )
                  Point3d( 0.2, 0.4, 0.0 )
                  Point3d( 0.2, 0.6, 0.0 )
                  Point3d( 0.0, 0.6, 0.0 )
                |])
            // MoNo.Graphics.IScene のインスタンス
            let scene =
              { new Graphics.IScene with
                  member __.Draw sc =
                    use scope = sc.Push()
                    scope.Lighting <- false
                    scope.Color <- System.Drawing.Color.Salmon
                    array.Draw sc // 頂点配列シーンの呼び出し
              }
            // シーンオブジェクトを登録
            con.Document.Val.Entries.Add (Wpf.Entry scene)
          } |> Wpf.Cont.toWpfCommand con None)

        // テクスチャマッピング
        MenuItem.New ("Texture Mapping",
          cont {
            let! path = con.Messenger.Send (Wpf.OpenFileMessage (Filter = "画像ファイル (*.bmp;*.jpg;*.png)|*.bmp;*.jpg;*.png"))

            // 画像ファイルの読込
            let bitmap = new System.Drawing.Bitmap( System.Drawing.Image.FromFile( path ) )

            // テクスチャオブジェクトを生成します。
            // ※ 注意！！ OpenGL 2.0 未満の場合はテクスチャのサイズは 2 のべき乗でなくてはなりません。
            //            詳しくは OpenGL の仕様を参照して下さい。
            let texture = MGL.CreateTextureObject( bitmap );

            // シーングラフにテクスチャを描画するシーンを登録します。
            let scene =
              { new Graphics.IScene with
                  member __.Draw sc =
                    use scope = sc.Push()
                    scope.Lighting <- false
                    scope.Color <- System.Drawing.Color.White
                    use binding = texture.Bind() // テクスチャをバインドします。
                    sc.DrawQuads (fun gl ->
                      gl.TexCoord( 0.0, 0.0 ); gl.Vertex( 0.0, 0.0, 0.0 )
                      gl.TexCoord( 1.0, 0.0 ); gl.Vertex( 1.0, 0.0, 0.0 )
                      gl.TexCoord( 1.0, 1.0 ); gl.Vertex( 1.0, 1.0, 0.0 )
                      gl.TexCoord( 0.0, 1.0 ); gl.Vertex( 0.0, 1.0, 0.0 ))
              }
            // シーンオブジェクトを登録
            con.Document.Val.Entries.Add (Wpf.Entry scene)
          } |> Wpf.Cont.toWpfCommand con None)
      ]
    [ MenuItem.Folder ("Samples", [ MenuItem.Folder ("OpenGL Samples", items) ]) ]

