﻿namespace QuickTemplate
open System
open System.ComponentModel.Composition
open System.Windows
open MoNo
open MoNo.Geometries
open MoNo.Studio
open MoNo.Ctrl


type [<Export (typeof<Studio.IMenuFactory>)>] private OperationMenuFactory() =
  inherit Studio.MenuFactory<QuickAppContext>()

  override __.CreateMenuItems con =
    let items =
      [
        MenuItem.New ("Click (simple)",
          cont {
            // クリックを1回検出します。
            let! e = con.ViewContext.Events.MouseClick |> Cont.ofObservable
            do! con.Messenger.Send (Wpf.InformationMessage (sprintf "Clicked: Location = %A" e.Location, "Click Sample"))
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("Click",
          cont {
            // クリックを1回検出します。クリック待ち状態のマウスカーソルを Corsors.Hand に設定しています。
            let! e = con.GetLButtonClick (fun op -> op.Cursor <- Forms.Cursors.Hand)
            do! con.Messenger.Send (Wpf.InformationMessage (sprintf "Clicked: Location = %A" e.Location, "Click Sample"))
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("Click Twice",
          cont {
            // クリックを2回検出します。
            let! e1 = con.GetLButtonClick ()
            let! e2 = con.GetLButtonClick ()
            do! con.Messenger.Send (Wpf.InformationMessage (sprintf "1st click = %A\n2nd click = %A" e1.Location e2.Location, "Click Twice Sample"))
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("Draw Line",
          cont {
            // 一点目のクリックを検出します。
            let! e1 = con.GetLButtonClick ()
            
            // 直線の両端点の座標を作成します。
            let p1 = e1.View.Camera.ScreenToWorld e1.Location
            let p2 = ref p1 // p2 はこの時点では p1 と同じ座標を設定しておきます

            // 二点目のクリックを検出します。
            let! e2 = con.GetLButtonClick (fun op ->
              op.MouseMove.Add (fun e -> p2 := e.View.Camera.ScreenToWorld e.Location; e.View.Invalidate())
              op.WorldScenes.Add (fun sc ->
                use scope = sc.Push()
                scope.Color <- Drawing.Color.Turquoise
                sc.DrawLines (fun gl -> gl.Vertices (p1, !p2))) |> ignore)

            // ２点 p1, p2 から Polyline3d オブジェクトを生成します。
            let pol = Polyline (false, [| p1; !p2 |])

            // エンティティを登録します。
            con.Document.Val.Entries.Add (Wpf.Entry pol)
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("Draw Polyline",
          cont {
            let mutable pol = Polyline.empty
            let mutable scene = GraphicsUT.CreateScene ignore
            let mutable cursorPos = Point3d.Zero

            // Operation オブジェクトを作ります
            let op = MoNo.Ctrl.Operation con.ViewContext

            // 現在作図中の折れ線を描画するシーンを登録します
            op.WorldScenes.Add (fun sc ->
              use scope = sc.Push()
              scope.Color <- Drawing.Color.Yellow
              scene.Draw sc
              if pol.Points.Length > 0 then
                sc.DrawLines (fun gl -> gl.Vertices (pol.Points.[pol.Points.Length - 1], cursorPos))) |> ignore

            // 現在のマウスカーソルの位置をワールド座標系の点として取得します
            op.MouseMove.Add (fun e ->
              cursorPos <- e.View.Camera.ScreenToWorld e.Location
              con.ViewContext.Invalidate ())

            // マウスカーソル位置の点を折れ線に追加します
            op.MouseClick.Add (fun _ ->
              pol <-
                Immutarray.append pol.Points (Immutarray [| cursorPos |])
                |> Polyline.ofImmutarray false
              scene <- Graphics.SceneFactory.NewInstance pol
              con.ViewContext.Invalidate ())

            // スペースキーで作図を終了します
            op.KeyDown.Add (fun e -> if e.KeyCode = Forms.Keys.Space then op.Exit ())

            // オペレーションを作動させます
            do! con.ByOperation op

            // 折れ線エンティティを登録します。
            if pol.Points.Length >= 2 then con.Document.Val.Entries.Add (Wpf.Entry pol)
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("Object Picking",
          cont {
            let! e = con.GetLButtonClick (fun op -> op.Cursor <- Forms.Cursors.Hand)
            let pick = e.View.PickAt<Wpf.Entry> (e.Location.ToPoint2i())
            if pick <> null then
              do! con.Messenger.Send (Wpf.InformationMessage (sprintf "HitObject = %A" pick.HitObject.Object, "Object Picking"))
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("HBall Handle Dragging",
          cont {
            // ボール状のハンドル（3Dビューに表示される、マウス操作可能なオブジェクト）を作成します。
            let ball = HBall( Point3d( 0.5, 0.5, 0.5 ) )

            // ボールハンドルの MouseDown イベントで、ボールをドラッグで移動するコードを記述します。
            ball.MouseDown.Add (fun e ->
              if e.Button = System.Windows.Forms.MouseButtons.Left && ball.IsHighlighted then
                let ballpos = ball.Position
                let mouseUp = LButtonUp( e.View )
                mouseUp.MouseMove.Add (fun e2 -> // マウスを離すまでの間、マウスドラッグによるボールの移動を計算します
                  let move = e.View.Camera.ScreenToWorld( e2.Location.ToPoint2i() - e.Location.ToPoint2i() )
                  ball.Position <- ballpos + move
                  e.View.Invalidate())
                e.Interrupt( mouseUp ) // ドラッグ操作のためのイベントハンドリングを「割り込み」させます
              )

            // HandleOperation に HBall ハンドルを設定してオペレーションを実行します
            do! con.ByOperation (HandleOperation (con.ViewContext, ball), ignore)
          } |> Wpf.Cont.toWpfCommand con None)
      ]
    [ MenuItem.Folder ("Samples", [ MenuItem.Folder ("Operation Samples", items) ]) ]
