﻿namespace CommandSample
open MoNo

type ViewModel() as this = 
  inherit ContRunner ()  // MoNo.ContRunner を継承

  // フィールドに単純なモデルオブジェクトを保持
  let model = Model()

  // モデルが持つ Value プロパティ（Reactive型）を公開。ビュー側で Label にバインドされる。
  member __.Value = model.Value

  // Value をインクリメントするコマンド
  // ボタンが押されると execute 引数に設定された関数が動作する。
  // MoNo.RAIL に用意された最もシンプルなコマンド生成。
  // ※ この方法によるコマンド生成では、このクラスが CommandContext を継承する必要はない。
  member val IncrementCommand =
    Wpf.Command.createWpfCommand None (fun _ -> model.Increment ())

  // Value をデクリメントするコマンド。
  // canExecute 引数が設定されており、Valueが1以上の場合のみボタンが有効になる。
  member val DecrementCommand =
    Wpf.Command.createWpfCommand (Some model.CanDecrement) (fun _ -> model.Decrement ())

  // Value を2倍にするコマンド。
  // MoNo.Wpf.Cont.toWpfCommand 関数を使うと、継続モナド cont からコマンドオブジェクトを生成出来る。
  // この関数は引数に CommandContext を要求するので、this を渡している点に注意。
  // （ただしこの例では継続モナドを使っている意味はない。マウスオペレーションやダイアログ操作、非同期処理などを行う際に活きてくる。）
  member val DoubleCommand =
    cont { model.Double() }
    |> Wpf.Cont.toWpfCommand this None

  // Value を半分にするコマンド。Valueが10以上のときのみ有効になる。
  // ここでは Model に定義された MoNo.Command オブジェクトから WPF のコマンドオブジェクトに
  // 変換する方式で定義されている。
  // 以下、簡単に MoNo.Command について説明。
  // コマンドとは、実行処理が記述された関数（execute）と、その処理が実行可能かどうかの状態（canExecute）を
  // セットにしたものだと考えると、この概念はモデル層でも有効に使えるはず。
  // しかし WPF のコマンドオブジェクトはビューへの依存関係があるためモデル層では使いづらい（使えない）。
  // そこでビューへの依存を廃したシンプルなコマンドオブジェクトとして MoNo.Command を定義し、それをビュー側で
  // WPFのコマンドオブジェクトに変換できるようにした。
  member val DivBy2Command =
    model.DivBy2Command
    |> Wpf.Command.toWpfCommand this

  // 非同期処理やプログレス通知付き処理を途中で中断するコマンド
  member val CancelCommand =
    Wpf.Command.createWpfCommand (Some this.IsInSession) this.AbortSession

  // 3秒間待って値を10倍にするコマンド
  member val TenTimesCommand =
    cont {
      let x = model.Value.Val
      let! y = this.ByAsync (async {
          do! Async.Sleep 3000
          return 10 * x
        })
      model.Value.Val <- y
    }
    |> Wpf.Cont.toWpfCommand this None


  member val TwentyTimesCommand =
    cont {
      let x = model.Value.Val
      let! y = this.ByProgress ( Progress.Make (fun token ->
          for i = 1 to 100 do
            System.Threading.Thread.Sleep 30
            token.Notify 0.01
          20 * x) )
      model.Value.Val <- y
    }
    |> Wpf.Cont.toWpfCommand this None